"""
Clicker HID Python.py

This is an example how to use the PIC32MX Clicker as HID USB device and Python 3 for datatransfer.

#---EQUIPMENT USED FOR THIS EXAMPLE---
Python USB Library: PyWinUSB
OS: Windows 8.1 64
PIC IDLE: MikroC Pro for PIC32 v3.5.0
PIC IDLE Extensions: USB Device Library from Libstock (http://www.libstock.com/projects/view/568/usb-device-library)

#---PYTHON / USB LIB INSTALLATION--- 
1. Install Python 3.4.3 or higher (enable the installers "PATH Option")
2. Install PyWinUSB
     Open Windows PowerShell (cmd.exe) by pushing WinButton + X
     enter "pip install pywinusb" in cmd.exe
3. Open Clicker HID Python.py with Python IDLE
4. Run py file by pushing F5

"""

import pywinusb.hid as hid
from time import sleep

VID = 0x04D8 #Microchip's Vendor ID
PID = 0x0100


class usbHandler(object):
    """Minimal code example for read write data from and to Clicker"""

    def __init__(self):
        """Init the first found Clicker as HID device"""
        self.allHIDs = hid.HidDeviceFilter(vendor_id=VID, product_id=PID)
        self.device = self.allHIDs.get_devices()[0]
        print(self.device.vendor_name, self.device.product_name )
        self.device.open()
        self.device.set_raw_data_handler(self.rxHandler) 

    def rxHandler(self, data):
        """Data transfered from Clicker is continues handled by this method"""
        print( "rx data:", data)

    def write(self, data64):
        """Easy write method for up to 64 bytes (U8 or Chars)"""
        # 1 byte (report id) + 64 byte report size
        if type(data64) == str: data64 = list( bytearray(data64,"utf-8"))
        report = self.device.find_output_reports()[0]
        txBuf = [0] + data64 
        txBuf += [0]*(65-len(txBuf))
        report.send(txBuf)

    def close(self):
        """Close method to set free the used usb resources""" 
        self.device.close()    
        

if __name__ == '__main__':
    """Send a string and a bytearray. rxHandler will show the echo"""
    usb=usbHandler()
    usb.write( "abcdefghi" )
    usb.write( [1,2,3,4,5] )
    sleep(5)
    usb.close()

