/*
 * Project name:
     HID Custom Demo
 * Date of creation
     1/12/2013
 * Description
     Example showing usage of USB custom HID class. Attach usb cable in order to connect
     development board to PC. After connection, the board is recognized as USB HID
     device. Open HID Terminal from Tools menu and select HID Custom Demo device. When
     sending data to device, data is echoed and result is displayed in the terminal
     window.
 * Test configuration:
     MCU:             P32MX795F512L
     dev.board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for PIC32
                      http://www.mikroe.com/mikroc/pic32/
     Notes:           Turn on USB PSW switch.
 */

#include <stdint.h>

// Buffer of 64 bytes
char buffer[64];
char sendBuf[64];

void USB1Interrupt() iv IVT_USB_1 ilevel 7 ics ICS_SRS{
  // Call library interrupt handler routine
  USBDev_IntHandler();
}

// Callback function which will be called on received packet
void DataReceived(uint8_t ep, uint16_t size){
  if(ep == 1){
    LATD.RD4 = ~LATD.RD4;
    // Send 64 bytes of data from buffer buff
    USBDev_SendPacket(1, buffer, 64);

    // Prepere buffer for reception of next packet
    USBDev_SetReceiveBuffer(1, buffer);
  }
}

void main(void){
  AD1PCFG = 0xFFFF;                        // Configure AN pins as digital I/O
  
  TRISD.RD1 = 0;
  LATD.RD1 = 1;
  TRISD.RD4 = 0;
  LATD.RD4 = 1;

  // Initialize HID Class
  USBDev_HIDInit();

  // Register callback function which will be called on received packet
  USBDev_RegisterDataReceivedHandler(DataReceived);

  // Initialize USB device module
  USBDev_Init();

  // Enable USB device interrupt
  USBIE_bit = 1;
  IPC11bits.USBIP = 7;
  EnableInterrupts();

  // Wait until device is configured (enumeration is successfully finished)
  while(USBDev_GetDeviceState() != _USB_DEV_STATE_CONFIGURED)
    ;

  // Set receive buffer where received data is stored
  USBDev_SetReceiveBuffer(1, buffer);

  // Infinite loop
  while(1){
     /*
     memset(&sendBuf[0],0,64);
     sendBuf[0] = 'G';
     sendBuf[1] = 'e';
     sendBuf[2] = 'i';
     sendBuf[3] = 'l';
     USBDev_SendPacket(1, SendBuf, 64);

     delay_ms(1000);
     */
  }

}